import {combineReducers, createStore} from "redux";

export class Command {
    constructor(typeName = this.constructor.name) {
        if (!typeName || typeName.length < 2) typeName = Command.createGuid();
        this.type = typeName;
    }

    asObject() {
        return JSON.parse(JSON.stringify(this));
    }

    handle(state) {
        console.warn('please implement a handle (state) method in your command ' + this.type);
        return state;
    }

    static createGuid() {
        return Command.fourChars() +
            Command.fourChars() + '-' +
            Command.fourChars() + '-' +
            Command.fourChars() + '-' +
            Command.fourChars() + '-' +
            Command.fourChars() +
            Command.fourChars() +
            Command.fourChars();
    }

    static fourChars() {
        return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
    }


}

export class CommandContainer {
    container = [];

    constructor(name) {
        this.name = name;
    }

    setDefaultState(defaultState) {
        this.defaultState = defaultState;
        return this;
    }

    register(commandInstance) {
        this.container.push(commandInstance.type);
        return this;
    }

    getReducer() {
        const keys = this.container;
        return (state = this.defaultState, action) => {
            if (!action) return state;
            if (!keys.includes(action.type)) return state;

            for (let i = 0; i < keys.length; i++) {
                const key = keys[i];
                if (key === action.type) {

                    const act = (state, action) => {
                        return action.command.handle(state);
                    };

                    return act(state, action);
                }
            }
            return state;
        };
    }

}

export class CommandExecutor {
    static _store;
    static _containers = [];

    static add(container) {
        this._containers.push(container);
    }

    static createStore() {
        const reducers = {};
        this._containers.map(x => {
            const reducer = x.getReducer();
            reducers[x.name] = reducer;
        });
        CommandExecutor._store = createStore(combineReducers(reducers));
        return CommandExecutor._store;
    }

    static getState() {
        return CommandExecutor._store.getState();
    }

    static execute(command) {

        let action = command.asObject();
        action.command = command;
        CommandExecutor._store.dispatch(action);
    }
}